<?php

namespace Sendeasy;

use \Throwable;

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit();
}

/**
 * SE_SOAP_Client class.
 *
 * Makes requests to the SendEasy SOAP API.
 */
class SE_SOAP_Client
{
    /**
     * Formats a request response from the API into an array with additional data.
     *
     * @access private
     * @param string $response
     * @return array
     */
    private static function se_handle_response(string $response, string $soap_action): array
    {
        $returned_response = ['response_status' => 'error'];

        switch ($response) {
            case '0':
                // A "0" response is indicates "success" for all requests except for QueryCreditBalance
                if ($soap_action === 'QueryCreditBalance') {
                    $returned_response['response_message'] = __('Zero credit. Please check whether your entered API credentials are correct and that you have sufficient credit.', 'sendeasy');
                } else {
                    $returned_response['response_message'] = __('OK.', 'sendeasy');
                    $returned_response['response_status'] = 'success';
                }
                break;
            case '-5':
                $returned_response['response_message'] = sprintf(__('A problem occured when sending a request to the %s. Please enable error logging to record and view errors in the %s logs.', 'sendeasy'), 'SendEasy API', 'WooCommerce');
                break;
            case '-10':
                $returned_response['response_message'] = __('Invalid login.', 'sendeasy');
                break;
            case '-25':
                $returned_response['response_message'] = __('Insufficient credit.', 'sendeasy');
                break;
            case '-40':
                $returned_response['response_message'] = __('Partially sent. Not all recipients could be sent the message.', 'sendeasy');
                break;
            case '-50':
                $returned_response['response_message'] = __('Error queuing SMS.', 'sendeasy');
                break;
            case '-60':
                $returned_response['response_message'] = __('SMS not found.', 'sendeasy');
                break;
            case '-70':
                $returned_response['response_message'] = __('Bulk SMS file could not be read.', 'sendeasy');
                break;
            case '-80':
                $returned_response['response_message'] = __('Bulk SMS file could not be saved.', 'sendeasy');
                break;
            default:
                $returned_response['response_status'] = 'success';
                $returned_response['response_message'] = $response;
        }

        $debug_log_enabled = get_option('se_enable_debug_log');

        // Do not log the "-5" display message for the user, as the actual cURL error is logged by the send_request method
        if (!empty($debug_log_enabled) && $debug_log_enabled === 'yes' && $response !== '-5') {
            wc_get_logger()->debug(
                "SendEasy {$soap_action} request made. Result: {$returned_response['response_status']}. Response: {$returned_response['response_message']}",
                ['source' => 'sendeasy-log']
            );
        }

        return $returned_response;
    }

    /**
     * Sends requests to the API.
     *
     * @access private
     * @param array $data
     * @return array
     */
    private static function se_send_request(array $data): array
    {
        $debug_log_enabled = get_option('se_enable_debug_log');

        $ch = curl_init();

        $data['xmlns'] = 'http://sms.netgen.co.za/';

        $xml = SE_XML_Handler::se_write_xml($data);

        $headers = [
            'Host: sms.netgen.co.za',
            'Accept: application/soap+xml',
            'Content-Type: application/soap+xml; charset=utf-8',
            'Content-Length: ' . strlen($xml)
        ];

        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_URL, 'http://sms.netgen.co.za/SMS_V2.asmx');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $curl_result = curl_exec($ch);

        $response = '-5';

        if (!$curl_result) {

            if (!empty($debug_log_enabled) && $debug_log_enabled === 'yes') {
                wc_get_logger()->error(
                    __CLASS__ . " cURL error when sending a {$data['soap_action']} request: "  . curl_error($ch),
                    ['source' => 'sendeasy-log']
                );
            }
        } else {

            if (!empty($debug_log_enabled) && $debug_log_enabled === 'yes') {
                wc_get_logger()->debug(
                    __CLASS__ . " cURL result returned after sending {$data['soap_action']} request: " . $curl_result,
                    ['source' => 'sendeasy-log']
                );
            }

            try {
                $response = SE_XML_Handler::se_read_xml($curl_result);
            } catch (Throwable $e) {
                if (!empty($debug_log_enabled) && $debug_log_enabled === 'yes') {
                    wc_get_logger()->error(
                        "An error occured when trying to read an expected XML response from the SendEasy API after sending a {$data['soap_action']} request: " . $e->getMessage(),
                        ['source' => 'sendeasy-log']
                    );
                }
            }
        }

        curl_close($ch);

        return self::se_handle_response($response, $data['soap_action']);
    }

    /**
     * Sends a QueryCreditBalance request request to the API.
     *
     * @access public
     * @param array $data
     * @return array
     */
    public static function se_query_credit_balance(array $data): array
    {
        $data['soap_action'] = 'QueryCreditBalance';

        $response = self::se_send_request($data);

        if ($response['response_status'] === 'success') {
            $response['response_message'] = SE_Formatter::se_format_decimal_number_string($response['response_message']);
        }

        return $response;
    }

    /**
     * Sends a SendSMS or SendSMSMulti request to the API.
     *
     * @access public
     * @param array $data
     * @param bool $multiple_recipients
     * @return array
     */
    public static function se_send_sms(array $data, bool $multiple_recipients = false): array
    {
        $data['soap_action'] = !$multiple_recipients ? 'SendSMS' : 'SendSMSMulti';

        if ($multiple_recipients) {
            foreach ($data['fields']['destination'] as &$destination) {
                $destination = "<string>{$destination}</string>";
            }
            $data['fields']['destination'] = implode($data['fields']['destination']);
        }

        $data['fields']['source'] = '27117810480';
        $data['fields']['deliveryDate'] = '';
        $data['fields']['mask'] = '';
        $data['fields']['trackingCode1'] = '';
        $data['fields']['trackingCode2'] = '';
        $data['fields']['trackingCode3'] = '';

        return self::se_send_request($data);
    }
}
